#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id: chembldb.py 318 2014-02-28 13:30:26Z cokelaer $

"""This module provides a class :class:`ChEMBL`

.. topic:: What is ChEMBL

    :URL:  https://www.ebi.ac.uk/chembldb/index.php/
    :REST: https://www.ebi.ac.uk/chembldb/index.php/ws
    :FAQS: https://www.ebi.ac.uk/chembldb/index.php/faq


    .. highlights::

        "Using the ChEMBL web service API users can retrieve data from the ChEMBL
        database in a programmatic fashion. The following list defines the currently
        supported functionality and defines the expected inputs and outputs of each
        method."

        -- From ChEMBL web page Dec 2012

"""
import os
from bioservices.services import REST
import webbrowser

__all__ = ["ChEMBL"]


class ChEMBL(REST):
    """Interface to `ChEMBL <http://www.ebi.ac.uk/chembldb/index.php>`_

    Here is a quick example to retrieve a target given its ChEMBL Id

    .. doctest::

        >>> from bioservices import ChEMBL
        >>> s = ChEMBL(verbose=False)
        >>> resjson = s.get_target_by_chemblId('CHEMBL240')
        >>> resjson['proteinAccession']
        'Q12809'

    By default, most methods return dictionaries (converted from json objects returned
    by the ChEMBL API), however, you can also set the format to be XML.

    """
    _url = "https://www.ebi.ac.uk/chemblws/"

    _chemblId_example = "CHEMBL1"
    _inChiKey_example = "QFFGVLORLPOAEC-SNVBAGLBSA-N"
    _smiles_example = "COc1ccc2[C@@H]3[C@H](COc2c1)C(C)(C)OC4=C3C(=O)C(=O)C5=C4OC(C)(C)[C@@H]6COc7cc(OC)ccc7[C@H]56"
    _smiles_struct_example = "C\C(=C/C=C/C(=C/C(=O)O)/C)\C=C\C1=C(C)CCCC1(C)C"
    _smiles_similar_example = _smiles_example
    _image_chemblId_example = "CHEMBL192"
    _bioactivities_example = "CHEMBL2"
    _target_chemblId_example = "CHEMBL2477"
    _target_uniprotId_example = "Q13936"
    _target_bioactivities_example = "CHEMBL240"
    _alt_compound_form_example = "CHEMBL278020"
    _target_approved_drugs_example = "CHEMBL1824"
    _target_refseq_example = 'NP_015325'
    _assays_example = "CHEMBL1217643"
    _inspect_example = ('CHEMBL1','compound')

    def __init__(self, verbose=False, cache=False):
        super(ChEMBL, self).__init__(url=ChEMBL._url,
            name="ChEMBL", verbose=verbose, cache=cache)

    def _process(self, query, frmt, request):
        self.devtools.check_param_in_list(frmt, ["json", "xml"])
        request = request + "." + frmt
        if isinstance(query, str):
            res = self.http_get(request % query, frmt=frmt)
        else:
            res = self.http_get([request % x for x in query], frmt=frmt)
        return res

    def _postprocess(self, data, key):
        try:
            return data[key]
        except:
            try:
                return [x[key] for x in data]
            except:
                return data

    def status(self):
        """Return the API status

        :return: Response is the string 'UP' if the service is running
        """
        res = self.http_get("status", frmt="json")
        try:
            #FIXME: wierd behaviour that is different on different systems...
            return res['status']
        except:
            return res

    def get_compounds_by_chemblId(self, query, frmt="json"):
        """Get compound by ChEMBLId

        :param query: a compound ChEMBLId or a list of those ones.
            if the identifier is invalid, the number 404 is returned.
        :param str frmt: json or xml (Default to json)
        :return: Compound Record (dictionary). If
            the query is a list of identifiers, a list of compound records is
            returned.

        If json format is requested, a dictionary is returned. Here are some of the keys:

         * acdLogd
         * chemblId
         * knownDrug
         * molecularFormula
         * molecularWeight
         * passesRuleOfThree
         * smiles
         * stdInChiKey

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print (s._chemblId_example)
            >>> resjson = s.get_compounds_by_chemblId(s._chemblId_example)

        """
        res = self._process(query, frmt, "compounds/%s")
        return res

    def get_compounds_by_chemblId_form(self, query, frmt="json"):
        """

        :param query: valid chembl identifier (or list)
        :param str frmt: json or xml (Default to json)

        See :meth:`get_compounds_by_chemblId` for full doc.

        ::

            >>> s.get_compounds_by_chemblId_form("CHEMBL2")
            [{u'chemblId': u'CHEMBL1347191', u'parent': False},
             {u'chemlId': u'CHEMBL1558', u'parent': False},
             {u'chemblId': u'CHEMBL2', u'parent': True}]

        """
        res = self._process(query, frmt, "compounds/%s/form")
        return self._postprocess(res, 'forms')

    def get_compounds_by_chemblId_drug_mechanism(self, query, frmt="json"):
        """

        :param query: valid chembl identifier (or list)
        :param str frmt: json or xml (Default to json)

        See :meth:`get_compounds_by_chemblId` for full doc.

        ::

            >>> s.get_compounds_by_chemblId_drug_mechanism("CHEMBL3")
            [{u'chemblId': u'CHEMBL1347191', u'parent': False},
            {u'chemblId': u'CHEMBL1558', u'parent': False},
            {u'chemblId': u'CHEMBL2', u'parent': True}]

        """
        res = self._process(query, frmt, "compounds/%s/drugMechanism")
        return res

    def get_individual_compounds_by_inChiKey(self, query, frmt="json"):
        """Get individual compound by standard InChi Key

        :param str query: a valid InChi key or a list.
        :param str frmt: json or xml (Default to json)
        :return: Compound Record as a dictionary (or list of dictionaries)

        In addition to the keys returned in :meth:`get_compounds_by_chemblId`,
        the following keys are returned:

         * acdAcidicPka
         * acdBasicPka
         * preferredCompoundName
         * species
         * synonyms

        .. doctest::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._inChiKey_example)
            QFFGVLORLPOAEC-SNVBAGLBSA-N
            >>> resjson = s.get_individual_compounds_by_inChiKey(s._inChiKey_example)
        """
        res = self._process(query, frmt, "compounds/stdinchikey/%s")
        return self._postprocess(res, 'compound')

    def get_compounds_by_SMILES(self, query, frmt="json"):
        """Get list of compounds by Canonical SMILES

        :param str query: a valid compound ChEMBLId or a list of those ones.
        :param str frmt: json or xml (Default to json)
        :return: Compound Record in XML or dictionary (if json requested). If
            query os a list/tuple, a  tuple of compound records is returned.

        If json format is requested, a dictionary is returned. The dictionary
        has a unique key 'compounds'. The value of that key is a list of compound
        records. For each compound record dictionary see :meth:`get_compounds_by_chemblId`.

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._smiles_example)
            >>> resjson = s.get_compounds_by_SMILES(s._smiles_example)
        """
        res = self._process(query, frmt, "compounds/smiles/%s")
        return res

    def get_compounds_containing_SMILES(self, query, frmt="json"):
        """Deprecated. Use get_compounds_substructure"""
        self.logging.error("Deprecated. Use get_compounds_substructure")

    def get_compounds_substructure(self, query, frmt="json"):
        """Get list of compounds containing the substructure represented
        by the given Canonical SMILES

        :param str query: a valid SMILES string or a list of those ones.
        :param str frmt: json or xml (Default to json)
        :return: see :meth:`get_compounds_by_SMILES`


        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._smiles_example)
            >>> res = s.get_compounds_substructure(s._smiles_struct_example)
        """
        res = self._process(query, frmt, "compounds/substructure/%s")
        return res

    def get_compounds_similar_to_SMILES(self, query, similarity=90, frmt="json"):
        """Get list of compounds similar to the one represented by the given Canonical SMILES.

        The similarity is at a cutoff percentage score (minimum value=70%, maximum value=100%).

        :param str query: a valid SMILES string or a list of those ones.
        :param str frmt: json or xml (Default to json)
        :return: Compound records. See :meth:`get_compounds_by_chemblId`

        Each dictionary has a set of keys amongst which:

         * acdAcidicPka
         * acdBasicPka
         * similarity
         * species

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._smiles_similar_example)
            >>> resjson = s.get_compounds_similar_to_SMILES(s._smiles_example, "100")
            >>> resjson = s.get_compounds_similar_to_SMILES(s._smiles_similar_example)
        """
        self.devtools.check_range(similarity, 70, 100)
        res = self._process(query, frmt, "compounds/similarity/%s/"+str(similarity))
        return self._postprocess(res, 'compounds')

    def get_image_of_compounds_by_chemblId(self, query, dimensions=500,
            save=True, view=True, engine="rdkit"):
        """Get the image of a given compound in PNG png format.

        :param str query: a valid compound ChEMBLId or a list/tuple of valid compound ChEMBLIds.
        :param int dimensions: optional argument. An integer z (:math:`1 \leq z \leq 500`)
            giving the dimensions of the image.
        :param save:
        :param view:
        :param engine: Defaults to rdkit. Implemented for the future but only one value for now.
        :param bool view: show the image. If True the images are opened.
        :return: the path (list of paths) used to save the figure (figures) (different from Chembl API)

        .. plot::
            :include-source:
            :width: 50%

            >>> from pylab import imread, imshow
            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> res = s.get_image_of_compounds_by_chemblId(s._image_chemblId_example)
            >>> imshow(imread(res['filenames'][0]))

        .. todo:: ignorecoords option
        """
        # NOTE: not async requests here.
        self.devtools.check_range(dimensions, 1,500)
        self.devtools.check_param_in_list(engine, ['rdkit'])
        queries = self.devtools.to_list(query)

        res = {'filenames':[], 'images':[], 'chemblids':[]}
        for query in queries:
            req = "compounds/%s/image" % query
            #url=self.url+'compounds/%s/image' % query
            if dimensions is not None:
                req += '?engine=%s&dimensions=%s'%(engine, dimensions)
            target_data = self.http_get(req, frmt=None)

            file_out = os.getcwd()
            file_out += '/%s.png' % query
            with open(file_out, "wb") as thisfile:
                thisfile.write(bytes(target_data))
                thisfile.close()
                self.logging.info("saved to %s" % file_out)

            fout = file_out
            res['chemblids'].append(query)
            res['filenames'].append(fout)
            res['images'].append(target_data)
        if view:
            webbrowser.open(res['filenames'][0])
        return res

    def get_compounds_activities(self, query, frmt="json"):
        """Get individual compound bioactivities

        :param str query: valid chembl identifier (or list)
        :param str frmt: json or xml (Default to json)
        :return: Bioactivity records in XML or dictionary (if json requested)

        If json format is requested, a dictionary is returned. Here are some
        of the keys:

         * activity_comment
         * assay_description
         * name_in_reference
         * organism
         * reference
         * target_confidence
         * target_name
         * units
         * value

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._bioactivities_example)
            >>> resjson = s.get_compounds_activities(s._bioactivities_example)
        """
        res = self._process(query, frmt, "compounds/%s/bioactivities")
        return res

    def get_target_by_chemblId(self, query, frmt="json"):
        """Get target by ChEMBLId

        :param str query: target ChEMBLId
        :param str frmt: json or xml (Default to json)
        :return: Target Record in XML or dictionary (if json requested)

        If json format is requested, a dictionary is returned. Here are
        some of dictionary's keys:

         * target
         * geneNames
         * organism
         * proteinAccession
         * targetType

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._target_chemblId_example)
            >>> resjson = s.get_target_by_chemblId(s._target_chemblId_example)
        """
        res = self._process(query, frmt, "targets/%s")
        return res

    def get_target_by_uniprotId(self, query, frmt="json"):
        """Get individual target by UniProt Accession Id

        :param str query: a valid uniprot accession Id.
        :param str frmt: json or xml (Default to json)
        :return: see :meth:`get_target_by_chemblId`

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._target_chemblId_example)
            >>> resjson = s.get_target_by_uniprotId(s._target_uniprotId_example)
        """
        res = self._process(query, frmt, "targets/uniprot/%s")
        if frmt == 'json':
            res = res['target']
        return res

    def get_target_by_refseq(self, query, frmt='json'):
        """Get individual target by RefSeq Accession identifier

        :param str query: a valid RefSeq accession Id
        :param str frmt: json or xml (Default to json)
        :return: Target Record

        .. warning:: this method works but I could not find any example to validate it.
            NP_001128722 provided on chembl blogspot does not work.
            NP_015325 from the ChEMBL API does not work either.

        """
        res = self._process(query, frmt, "targets/refseq/%s")
        return res

    def get_target_bioactivities(self, query, frmt="json"):
        """Get individual target bioactivities

        :param str query: a valid target ChEMBLId.
        :param str frmt: json or xml (Default to json)

        :return: see :meth:`get_compounds_activities`

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._target_bioactivities_example)
            >>> resjson = s.get_target_bioactivities(s._target_bioactivities_example)
        """
        res = self._process(query, frmt, "targets/%s/bioactivities")
        return self._postprocess(res, 'bioactivities')

    def get_all_targets(self, frmt="json"):
        """Get all targets in a dictionary

        :param str frmt: json or xml (Default to json)
        :return: Target Record in a dictionary (default is json)

        The returned list contains a dictionary for each **target**.
        Here are some keys contained in the dictionaries:

          * chemblId
          * description
          * geneNames
          * organism
          * preferredName
          * proteinAccession
          * synonyms
          * targetType

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> res = s.get_all_targets()
        """
        self.devtools.check_param_in_list(frmt, ["json", "xml"])
        res = self.http_get("targets." + frmt, frmt=frmt)
        return self._postprocess(res, 'targets')

    def get_assays_by_chemblId(self, query, frmt="json"):
        """Get assay by ChEMBLId

        :param str query: a valid assay ChEMBLId.
        :param str frmt: json or xml (Default to json)
        :return: Assay record as a dictionary (if json requested)
            (list of dictionaries if input is a list)

        If json format is requested, a dictionary is returned.
        with some of the following keys:

         * assayDescription
         * assayOrganism
         * assayStrain
         * assayType
         * chemblId
         * journal
         * numBioactivities

        .. doctest::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._assays_example)
            CHEMBL1217643
            >>> resjson = s.get_assays_by_chemblId(s._assays_example)

        """
        res = self._process(query, frmt, "assays/%s")
        return res

    def get_assays_bioactivities(self, query, frmt="json"):
        """Get individual assay bioactivities

        :param str query: a valid assay ChEMBLId.
        :param str frmt: json or xml (Default to json)
        :return: Bioactivity records for a given assay (dictionary)

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._assays_example)
            >>> resjson = s.get_assays_bioactivities(s._assays_example)
        """
        res = self._process(query, frmt, "assays/%s/bioactivities")
        return res

    def get_alternative_compound_form(self, query, frmt="json"):
        """ Get list of alternative compound forms (e.g. parent and salts) of compound

        :param query: a valid compound ChEMBLId
        :param frmt: json or xml (Default to json)
        :return: List of ChEMBLIDs which correspond to alternative forms of query compound

        >>> from bioservices import *
        >>> s = ChEMBL(verbose=False)
        >>> resjson = s.get_alternative_compound_form(s._alt_compound_form_example)
        """

        res = self._process(query, frmt, "compounds/%s/form")
        return res

    def get_approved_drugs(self, query, frmt="json"):
        """Get list of approved drugs chembl compound details

        :param query: a valid target ChEMBLId
        :param frmt: json or xml (Default to json)
        :return: list of approved drugs ChEMBL compound details (dictionary)

        >>> from bioservices import *
        >>> s = ChEMBL(verbose=False)
        >>> resjson = s.get_approved_drugs(s._target_approved_drugs_example)
        """
        res = self._process(query, frmt, "targets/%s/approvedDrug")
        return res

    def inspect(self, query, item_type):
        """Open the URL of a query in a browser.

        :param str query: a valid ChEMBLId of a compound, target or assay.
        :param str item_type: a valid type. Might be compound, target or assay

        ::

            >>> from bioservices import *
            >>> s = ChEMBL(verbose=False)
            >>> print(s._assays_example)
            >>> s.inspect(*s._inspect_example)
            >>> print(s._assays_example)
            >>> s.inspect(s._assays_example,'assay')

        """
        url = "https://www.ebi.ac.uk/chembldb/%s/inspect/%s"%(item_type, query)
        webbrowser.open(url)

    def version(self):
        """Return version of the API on the server"""
        res = self.http_get("status", frmt="json")
        try:
            #FIXME: wierd behaviour that is different on different systems...
            return res['version']
        except:
            return res


class BenchmarkChembl(ChEMBL):

    def __init__(self):
        super(BenchmarkChembl, self).__init__(cache=True)

    def benchmark(self, N=1000):
        #self.name = "compounds"
        import time
        t1 = time.time()
        res = self.get_compounds_by_chemblId(['CHEMBL%s' % i for i in range(1,N)])
        t2 = time.time()
        res1 = t2-t1
        print(t2-t1)

        t1 = time.time()
        res = [self.get_compounds_by_chemblId('CHEMBL%s' % i) for i in range(1,N)]
        t2 = time.time()
        res2   = t2-t1
        print(t2-t1)

        return res, res1, res2

    def benchmark2(self, N):
        print("ORIGINAL chembl")
        from chembl_webresource_client import CompoundResource
        compounds = CompoundResource()
        import time

        t1 = time.time()
        res = compounds.get(['CHEMBL%s' % x for x in range(1,N)])
        t2 = time.time()

        res1 = t2-t1
        print(t2-t1)

        t1 = time.time()
        res = [compounds.get('CHEMBL%s' % x) for x in range(1,N)]
        t2 = time.time()

        res2 = t2-t1
        print(t2-t1)
        return res, res1, res2


    def compare_benchmark(self,n=10, N=1000):

        df = {'bs_sync':[], 'bs_normal':[], 'ch_sync':[], 'ch_normal':[]}
        for i in range(0,n):
            print(i)
            res, res1, res2 = self.benchmark(N)
            df['bs_sync'].append(res1)
            df['bs_normal'].append(res2)
            res, res1, res2 = self.benchmark2(N)
            df['ch_sync'].append(res1)
            df['ch_normal'].append(res2)
            print
        import pandas as pd
        df = pd.DataFrame(df)
        import pylab
        pylab.clf()
        df  = df[['bs_normal','ch_normal','bs_sync','ch_sync']]
        pylab.axvline(2.5)
        df.boxplot()
        return df



from bioservices import BioGRID

from nose.plugins.attrib import attr

# IF BIOMART WORKS, BIOGRIOD should work


# BioGRID design is not meant to be used really as a class. Need to be fixed.
#class test_biodbnet(object):
#    @classmethod
#    def setup_class(klass):
#        klass.s = BioGRID(verbose=False)


@attr('slow')
def test_biogrid():
    b = BioGRID(query=["map2k4","akt1"],taxId = "9606")
    b.biogrid.interactors
    b = BioGRID(query=["mtor","akt1"],taxId="9606",exP="two hybrid")
    b = BioGRID(query="mtor",taxId="9606")
    b.biogrid.interactors





# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""Interface to the PDB web Service.

.. topic:: What is PDB ?

    :URL: http://www.rcsb.org/pdb/
    :REST: http://www.rcsb.org/pdb/software/rest.do

    .. highlights::

        An Information Portal to Biological Macromolecular Structures

        -- PDB home page, Feb 2013

    :Status: in progress not for production

"""
from __future__ import print_function

from bioservices.services import REST

__all__ = ["PDB"]


class PDB(REST):
    """Interface to part of the `PDB <http://www.rcsb.org/pdb>`_ service

    :Status: in progress not for production. You can get all ID and retrieve
        uncompressed file in PDB/FASTA formats for now. New features will be
        added on request.

    .. doctest::

        >>> from bioservices import PDB
        >>> s = PDB()
        >>> res = s.get_file("1FBV", "pdb")

    """

    def __init__(self, verbose=False, cache=False):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages (default is off)

        """
        url="http://www.rcsb.org/pdb"
        super(PDB, self).__init__(name="PDB", url=url, verbose=verbose,
            cache=cache)

    def search(self, query):
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <orgPdbQuery>
        <version>B0907</version>
        <queryType>org.pdb.query.simple.ExpTypeQuery</queryType>
        <description>Experimental Method Search : Experimental Method=SOLID-STATE NMR</description>
        <mvStructure.expMethod.value>SOLID-STATE NMR</mvStructure.expMethod.value>
        </orgPdbQuery>
        """
        res = self.http_post("search", frmt="", data=query)
        return res

    def get_current_ids(self):
        """Get a list of all current PDB IDs."""
        res = self.http_get("rest/getCurrent", frmt="xml")
        res = self.easyXML(res)
        res = [x.attrib['structureId'] for x in res.getchildren()]
        return res

    def get_file(self, identifier, frmt, compression=False, headerOnly=False):
        """Download a file in a specified format

        :param int identifier: a valid Identifier. See :meth:`get_current_ids`.
        :param str fileFormat: a valid format in "pdb", "cif", "xml"

        .. doctest::

            >>> from bioservices import PDB
            >>> s = PDB()
            >>> res = s.get_file("1FBV", "pdb")
            >>> import tempfile
            >>> fh = tempfile.NamedTemporaryFile()
            >>> fh.write(res)
            >>> # manipulate the PDB file with your favorite tool
            >>> # close the file ONLY when finished (this is temporary file)
            >>> # fh.close()

        reference: http://www.rcsb.org/pdb/static.do?p=download/http/index.html
        """
        valid_formats = ["pdb", "cif", "xml"]
        self.devtools.check_param_in_list(frmt, valid_formats)
        self.devtools.check_param_in_list(headerOnly, [True, False])
        if headerOnly is True:
            headerOnly = "YES"
        else:
            headerOnly = "NO"

        query = "files/" + identifier + "." + frmt
        if compression is True:
            query += ".gz"

        params = {'headerOnly': headerOnly}

        if frmt == "xml":
            res = self.http_get(query, frmt=frmt, params=params)
            if compression is False:
                res = self.easyXML(res)
        else:
            res = self.http_get(query, frmt="txt", params=params)
        return res

    def get_ligands(self, identifier):
        """List the ligands that can be found in a PDB entry

        :param identifier: a valid PDB identifier (e.g., 4HHB)
        :return: xml document


            >>> from bioservices import PDB
            >>> s = PDB()
            >>> s.get_ligands("4HHB")

        Then, ::

            x = s.get_ligands("4HHB")
            from pyquery import PyQuery as pq
            d = pq(x)


        """

        res = self.http_get("rest/ligandInfo", frmt='xml',
                params={'structureId': identifier})
        return res


    def get_xml_query(self, query):
        """Send an XML query

        query = '<?xml version="1.0" encoding="UTF-8"?>
        <orgPdbQuery>
        <version>B0907</version>
        <queryType>org.pdb.query.simple.ExpTypeQuery</queryType>
        <description>Experimental Method Search : Experimental Method=SOLID-STATE NMR</description>
        <mvStructure.expMethod.value>SOLID-STATE NMR</mvStructure.expMethod.value>
        </orgPdbQuery>
        '
        """
        res = self.http_post("query/post",
                data=query,
                headers=self.get_headers(content='default'))
        return res






from bioservices import PSICQUIC, psicquic
from nose.plugins.attrib import attr


@attr('slow')
class test_psicquic(object):
    @classmethod
    def setup_class(klass):
        klass.s = PSICQUIC(verbose=False)


    def test_read_registry(self):
        self.s.read_registry()

    def test_print_status(self):
        self.s.print_status()

    def test_registry(self):
        N1 = len(self.s.registry_names)
        N2 = len(self.s.registry_versions)
        N2 = len(self.s.registry_restricted)
        assert N1 == N2

    def test_query(self):
        if 'intact' not in self.s.activeDBs or 'matrixdb' not in self.s.activeDBs:
            return

        self.s.query("intact", "brca2", "tab27")
        self.s.query("intact", "zap70", "xml25")
        self.s.query("matrixdb", "*", "xml25")
        try:
            self.s.query("matxdb", "*", "xml25")
            assert False
        except:
            assert True

        self.s.query("matrixdb", "*", "xml25", firstResult=10, maxResults=10)

        # accessing the string DB
        if 'string' in self.s.activeDBs:
            self.s.query("string", "species:10090", firstResult=0, maxResults=100, output="tab25")

@attr('fixme')
def test_appsPPI():
    p = psicquic.AppsPPI(verbose=False)
    p.queryAll("ZAP70", ["intact"])
    p.summary()




#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id: chembldb.py 318 2014-02-28 13:30:26Z cokelaer $

"""This module provides a class :class:`ENA`

.. topic:: What is ENA

    :URL:  https://www.ebi.ac.uk/ena


    .. highlights::

        The European Nucleotide Archive (ENA) provides a comprehensive 
        record of the world's nucleotide sequencing information, covering 
        raw sequencing data, sequence assembly information and functional 
        annotation. 

        -- From ENA web page Jan 2016

.. addedversion:: 1.4.4

"""
import os
from bioservices.services import REST
import webbrowser

__all__ = ["ENA"]


class ENA(REST):
    """Interface to `ChEMBL <http://www.ebi.ac.uk/ena/index.php>`_

    Here is a quick example to retrieve a target given its ChEMBL Id

    .. doctest::

        >>> from bioservices import ChEMBL
        >>> s = ENA(verbose=False)


    Retrieve read domain metadata in XML format::
        
        print(e.get_data('ERA000092', 'xml'))

    Retrieve assemble and annotated sequences in fasta format::

        print(e.get_data('A00145', 'fasta'))

    The range parameter can be used in combination to retrieve a subsequence 
    from sequence entry A00145 from bases 3 to 63 using ::

        e.get_data('A00145', 'fasta', fasta_range=[3,63])

    Retrieve assembled and annotated subsequences in HTML format (same 
    as above but in HTML page).

        e.view_data('A00145')


    Retrieve expanded CON records:

    To retrieve expanded CON records use the expanded=true parameter. For 
    example, the expanded CON entry AL513382 in flat file format can be i
    obtained as follows::

        e.get_data('AL513382', frmt='text', expanded=True)

    Expanded CON records are different from CON records in two ways. 
    Firstly, the expanded CON records contain the full sequence in addition 
    to the contig assembly instructions. Secondly, if a CON record contains 
    only source or gap features the expanded CON records will also display 
    all features from the segment records.

    Retrieve assembled and annotated sequence header in flat file format

    To retrieve assembled and annotated sequence header in flat file 
    format please use the header=true parameter, e.g.:

        e.get_data('BN000065', 'text', header=True)


    Retrieve assembled and annotated sequence records using sequence 
    versions::

        e.get_data('AM407889.1', 'fasta')
        e.get_data('AM407889.2', 'fasta')
    


    .. todo:: Taxon viewer, image retrieval.

    """
    _url = "http://www.ebi.ac.uk/ena/data/view"


    def __init__(self, verbose=False, cache=False):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        super(ENA, self).__init__(name="ENA", url=ENA._url,
                verbose=verbose, cache=cache)
        self.TIMEOUT = 100

    def get_data(self, identifier, frmt, fasta_range=None, expanded=None,
            header=None):
        # somehow the params param does not work, we need to construct the
        # entire url
        url = self.url + '/' + identifier
        assert frmt in ['fasta', 'xml', 'text']
        url += "&display=%s" % frmt

        if fasta_range is not None:
            url += "&range=%s-%s" % (fasta_range[0], fasta_range[1])

        if expanded is not None and expanded is True:
            url += "&expanded=true"

        if header is not None and header is True:
            url += "&header=true"


        res = self.http_get(url)
        res = res.content
        return res

    def view_data(self, identifier, fasta_range=None):
        url = self.url + '/' + identifier
        if fasta_range is not None:
            url += "&range=%s-%s" % (fasta_range[0], fasta_range[1])
        self.on_web(url)








"""BioServices

import bioservices
u = bioservices.uniprot.UniProt()
u.search("ZAP70")


see online documentation for details on pypi or github.

"""
#from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

import pkg_resources
__version__ = "$Id$$, $Rev$"
try:
    version = pkg_resources.require("bioservices")[0].version
    __version__ = version
except:
    version = __version__

try:
    # This is not striclty speaking required to run and use bioservices
    # However, some functions and python notebooks included in bioservices do
    import pandas as pd
except:
    print("BioServices %s warning: pandas is not installed on your system." % version)
    print("Some features requires this library and future version of BioServices may use it.")


# Initialise the config directory if not already done
from easydev import CustomConfig
configuration = CustomConfig("bioservices", verbose=False)
bspath = configuration.user_config_dir



# Add bioservices.uniprot to sys.modules to prevent cycles in our imports
#import bioservices.uniprot
#bioservices.uniprot  # Stop flake8 error


from . import settings
from .settings import *

from . import services
from .services import *

from . import biomodels
from .biomodels import *

from . import chebi
from .chebi import *

from . import ena
from ena import ENA

from . import ensembl
from .ensembl import *

from . import geneprof
from .geneprof import *

from . import kegg
from .kegg import *

from . import hgnc
from .hgnc import *

from . import intact
from .intact import *

from . import rhea
from .rhea import *

from . import xmltools
from .xmltools import *

from . import wikipathway
from .wikipathway import *

from . import pdb
from .pdb import *

from . import pride
from .pride import *

from . import uniprot
from .uniprot import *

from . import unichem
from .unichem import *

from . import wsdbfetch
from .wsdbfetch import *


from . import reactome
from .reactome import *

from . import quickgo
from .quickgo import *


from . import chembl
from .chembl import *

from . import picr
from .picr import *

from . import psicquic
from .psicquic import *

from . import ncbiblast
from .ncbiblast import *

#from . import readseq
#from .readseq import *

from . import biogrid
from .biogrid import *

from . import miriam
from .miriam import *

from . import arrayexpress
from .arrayexpress import *

from . import biomart
from .biomart import *

from . import eutils
from .eutils import *

from . import pathwaycommons
from .pathwaycommons import *


from . import muscle
from .muscle import *

from . import biodbnet
from .biodbnet import *

from . import biocarta
from .biocarta import *

# sub packages inside bioservices.

#import mapping
from . import apps
#import dev


from . import clinvitae
from .clinvitae import Clinvitae



from bioservices import BioDBNet
from nose.plugins.attrib import attr

@attr('skip_travis')
class test_biodbnet(object):
    @classmethod
    def setup_class(klass):
        klass.s = BioDBNet(verbose=False)

    def test_db2db(self):
        self.s.db2db("UniProt Accession", "Gene ID", "P43403")



    def test_dbfind(self):
        inputValues = 'ENSG00000121410, ENSG00000171428'
        self.s.dbFind("Ensembl Gene ID", inputValues)

    @attr('slow')
    def test_dbreport(self):
       self.s.dbReport("UniProt Accession", "P43403")

    def test_dbwald(self):
        self.s.dbWalk('Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID', "ENSG00000121410", 9606)







from bioservices import UniChem
from nose.plugins.attrib import attr


@attr('skip_travis')
class test_UniChem(UniChem):
    def __init__(self):
        super(test_UniChem, self).__init__(verbose=False, cache=False)

    def test_get_compound_ids_from_src_id(self):
        res1 = self.get_compound_ids_from_src_id("CHEMBL2", "chembl", "chebi")
        res2 = self.get_compound_ids_from_src_id(["CHEMBL2"], "chembl", "chebi")
        assert res1 == res2[0]
        assert res1 == [{u'src_compound_id': u'8364'}]
        assert res2[0] == [{u'src_compound_id': u'8364'}]

    def test_get_all_src_ids(self):
        assert len(self.get_all_src_ids())>=23

    def test_get_source_id(self):
        assert self._get_source_id("chembl") == 1
        assert self._get_source_id("1") == 1
        assert self._get_source_id(1) == 1

        try:
            self._get_source_id("wrong")
            assert False
        except:
            assert True

        try:
            self._get_source_id("20000")
            assert False
        except:
            assert True

    def test_get_src_information(self):
        assert self.get_source_information("chebi")['name'] == "chebi"

        assert self.get_source_information(['chembl', 'drugbank'])[0]['name']=="chembl"

    def test_get_all_compound_ids_from_src_id(self):
        res = self.get_all_compound_ids_from_all_src_id("CHEMBL12", "chembl")
        res = self.get_all_compound_ids_from_all_src_id("CHEMBL12", "chembl", "chebi")


    def test_mapping(self):
        res = self.get_mapping("kegg_ligand", "chembl")
        assert len(res)>0

    def test_get_src_compound_ids_from_inchikey(self):
        self.get_src_compound_ids_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
        self.get_src_compound_ids_from_inchikey(["AAOVKJBEBIDNHE-UHFFFAOYSA-N"])
        #self.get_src_compound_ids_all_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
        #self.get_src_compound_ids_all_from_inchikey(["AAOVKJBEBIDNHE-UHFFFAOYSA-N"])

    def test_structure(self):
        self.get_structure("CHEMBL12", "chembl")
        self.get_structure(["CHEMBL12"], "chembl")
        self.get_structure_all("CHEMBL12", "chembl")


    def test_get_src_compound_id_url(self):
        self.get_src_compound_id_url("CHEMBL12", "chembl", "drugbank")
        self.get_src_compound_id_url(["CHEMBL12"], "chembl", "drugbank")


    def test_get_src_compound_ids_all_from_obsolete(self):
        self.get_src_compound_ids_all_from_obsolete("DB07699", "2")[0]
        self.get_src_compound_ids_all_from_obsolete("DB07699", "2", "2")[0]

    def test_get_verbose_src_compound_ids_from_inchikey(self):
        assert self.get_verbose_src_compound_ids_from_inchikey("GZUITABIAKMVPG-UHFFFAOYSA-N") != 400
        assert self.get_verbose_src_compound_ids_from_inchikey("QFFGVLORLPOAEC-SNVBAGLBSA-N") != 400 

    @attr('slow')
    def test_get_auxiliary_mapping(self):
        self.settings.TIMEOUT = 100
        res = self.get_auxiliary_mappings(1)
        assert len(res)>0




from bioservices import BioModels
from nose.plugins.attrib import attr

modelId = 'BIOMD0000000256'
uniprotId = 'P10113'
pubId = '18308339'
GOId = 'GO:0001756'
reacID = "REACT_1590"
personName = "LeNovere"


class test_biomodels(object):

    @classmethod
    def setup_class(klass):
        klass.s = BioModels(verbose=False)

    def test_size(self):
        L1 = len(self.s.getAllCuratedModelsId())
        L2 = len(self.s.getAllNonCuratedModelsId())
        L = len(self.s)
        assert L == L1+L2

    def test_getAllModelsId(self):
        assert len(self.s.getAllModelsId()) > 800

    def test_getAllCuratedModelsId(self):
        assert len(self.s.getAllCuratedModelsId()) > 100

    def test_getAllNonCuratedModelsId(self):
        assert len(self.s.getAllNonCuratedModelsId()) > 100

    def test_getModelById(self):
        self.s.getModelById('MODEL1006230101')

    def test_getModelSBMLById(self):
        self.s.getModelSBMLById(modelId)
        self.s.getModelSBMLById('MODEL1006230101')

    def test_getAuthorsByModelId(self):
        res = self.s.getAuthorsByModelId(modelId)
        assert res == ['Rehm M', 'Huber HJ', 'Dussmann H', 'Prehn JH']

    def test_getDateLastModifByModelId(self):
        res = self.s.getDateLastModifByModelId(modelId)
        assert res == '2012-05-16T14:44:17+00:00'

    def test_getEncodersByModelId(self):
        res = self.s.getEncodersByModelId("BIOMD0000000256")
        assert res == ['Lukas Endler']

    def test_getLastModifiedDateByModelId(self):
        res = self.s.getLastModifiedDateByModelId("BIOMD0000000256")
        assert res == '2012-05-16T14:44:17+00:00'

    def test_getModelNameById(self):
        res = self.s.getModelNameById("BIOMD0000000256")
        assert res == 'Rehm2006_Caspase'

        try:
            self.s.getModelNameById("dummy")
            assert False
        except:
            assert True

    def test_getModelsIdByChEBI(self):
        res = self.s.getModelsIdByChEBI('CHEBI:4978')
        res == ['BIOMD0000000217', 'BIOMD0000000404']

    def test_getModelsIdByChEBIId(self):
        res = self.s.getModelsIdByChEBIId('CHEBI:4978')
        assert res == ['BIOMD0000000404']

    def test_getSimpleModelsByChEBIIds(self):
        self.s.getSimpleModelsByChEBIIds('CHEBI:4978')

    @attr('slow')
    def test_getSimpleModelsRelatedWithChEBI(self):
        res = self.s.getSimpleModelsRelatedWithChEBI()
        from bioservices import xmltools
        res = xmltools.easyXML(res.encode('utf-8'))
        modelIDs = set([x.findall('modelId')[0].text for x in res.getchildren()])
        assert len(modelIDs) > 1

    def test_getPublicationByModelId(self):
        res = self.s.getPublicationByModelId("BIOMD0000000256")
        assert res == '16932741'

    def test_getSimpleModelByIds(self):
        self.s.getSimpleModelsByIds(modelId)

    def test_getModelsIdByPerson(self):
        self.s.getModelsIdByPerson(personName)

    def test_getSimpleModelsByReactomeIds(self):
        return self.s.getSimpleModelsByReactomeIds(reacID)

    def test_getModelsIdByUniprotId(self):
        return self.s.getModelsIdByUniprotId(uniprotId)

    def test_getModelsIdByUniprotIds(self):
        self.s.getModelsIdByUniprotIds(["P10113", "P10415"])

    def test_getModelsIdByName(self):
        return self.s.getModelsIdByName('2009')

    def test_getModelsIdByPublication(self):
        res = self.s.getModelsIdByPublication(pubId)
        assert res == ['BIOMD0000000201']

    def test_getModelsIdByGO(self):
        return self.s.getModelsIdByGO(GOId)

    def test_getModelsIdByTaxonomy(self):
        return self.s.getModelsIdByTaxonomy("EGF")

    def test_getModelsIdByTaxonomyId(self, taxonomyId='9606'):
        return self.s.getModelsIdByTaxonomyId(taxonomyId)

    def test_getSubModelSBML(self):
        self.s.getSubModelSBML("BIOMD0000000242", "cyclinEdegradation_1")

    def test_getModelsIdByGOId(self):
        self.s.getModelsIdByGOId(GOId)

    def test_extra_getChEBIIds(self):
        self.s.extra_getChEBIIds(99, 101)
        try:
            self.s.extra_getChEBIIds(1000, 101)
            assert False
        except:
            assert True

    def test_extra_getReactomeIds(self):
        self.s.extra_getReactomeIds(99, 101)  # just to cross the 100 Ids
        self.s.extra_getReactomeIds(89, 90)  # just to get one output REACT_89
        try:
            self.s.extra_getReactomeIds(1000, 101)
            assert False
        except:
            assert True

    def test_extra_getUniprotIds(self):
        self.s.extra_getUniprotIds(11099, 11101)
        self.s.extra_getUniprotIds(10113, 10114)

        try:
            self.s.extra_getUniprotIds(1000, 101)
            assert False
        except ValueError:
            assert True

    def test_getModelsIdByUniprot(self):
        self.s.getModelsIdByUniprot("P10113")



from bioservices import ChEBI





def test_chebi():
    ch = ChEBI()
    ch.getCompleteEntity("CHEBI:10102")
    res = ch.conv("CHEBI:10102", "KEGG COMPOUND accession")
    assert res == ['C07484']

    try:
        res = ch.conv("CHEBI:10102", "wrong db")
        assert False
    except:
        assert True

    ch.getOntologyChildren("CHEBI:27732")
    ch.getOntologyParents("CHEBI:27732")
    ch.getUpdatedPolymer("CHEBI:27732")

def test_chebi_mass():
    ch = ChEBI()
    mass1 = ch.getCompleteEntity("CHEBI:27732").mass
    assert float(mass1) == 194.19076

    res = ch.getLiteEntity("194.19076", "MASS", 5, 2)
    assert  res[0]["chebiId"] == "CHEBI:27732"

    # should return nothing
    res = ch.getLiteEntity("194.19076", "SMILES", 5, 2)

def test_polymer():
    ch = ChEBI()
    x = ch.serv.getUpdatedPolymer("CHEBI:27732")
    x.chebiId
    #Out[14]: 27732
    x.globalCharge
    #0
    x.globalFormula
    # C8H10N4O2
    x.updatedStructure

def test_completelist():
    ch = ChEBI()
    names = [x.chebiAsciiName for x in ch.getCompleteEntityByList(["CHEBI:27732","CHEBI:36707"])]

    names = [str(x) for x in names]
    assert names == ["caffeine", "2-acetyl-1-alkyl-sn-glycero-3-phosphocholine"]

def test_search():
    ch = ChEBI()
    smiles = ch.getCompleteEntity("CHEBI:27732").smiles
    ch.serv.getStructureSearch(smiles, "SMILES", "SIMILARITY", 3, 0.25)

def test_ontology():
    ch = ChEBI()
    ch.getAllOntologyChildrenInPath("CHEBI:27732", "has part")

def test_structure():
    ch = ChEBI()
    smiles = ch.getCompleteEntity("CHEBI:27732").smiles
    ch.getStructureSearch(smiles, "SMILES", "SIMILARITY", 3, 0.25)




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id$
"""This module provides a class :class:`~PathwayCommons`

.. topic:: What is PathwayCommons ?

    :URL: http://www.pathwaycommons.org/about
    :REST:

    .. highlights::

        Pathway Commons is a convenient point of access to biological pathway
        information collected from public pathway databases, which you can
        search, visualize and download. All data is freely available, under the
        license terms of each contributing database.

       -- PathwayCommons home page, Nov 2013


Data is freely available, under the license terms of each contributing database.

"""
from __future__ import print_function

from bioservices.services import REST, BioServicesError


__all__ = ["PathwayCommons"]

class PathwayCommons(REST):
    """Interface to the `PathwayCommons <http://www.pathwaycommons.org/about>`_ service


    >>> from bioservices import *
    >>> pc2 = PathwayCommons(verbose=False)
    >>> res = pc2.get("http://identifiers.org/uniprot/Q06609")


    """

    #: valid formats
    _valid_format = ["GSEA", "SBGN", "BIOPAX", "BINARY_SIF", "EXTENDED_BINARY_SIF"]
    _valid_direction = ["BOTHSTREAM", "DOWNSTREAM", "UPSTREAM"]
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages

        """
        # Note that the url must end with / character to be reachable...
        super(PathwayCommons, self).__init__(name="PathwayCommons",
                url="http://www.pathwaycommons.org/pc2/", verbose=verbose)
        self.easyXMLConversion = False


        self._default_extension = "json"

    # just a get/set to the default extension
    def _set_default_ext(self, ext):
        self.devtools.check_param_in_list(ext, ["json","xml"])
        self._default_extension = ext
    def _get_default_ext(self):
        return self._default_extension
    default_extension = property(_get_default_ext, _set_default_ext,
             doc="set extension of the requests (default is json). Can be 'json' or 'xml'")

    def search(self, q, page=0, datasource=None, organism=None, type=None):
        """Text search in PathwayCommons using Lucene query syntax

        Some of the parameters are BioPAX properties, others are composite
        relationships.

        All index fields are (case-sensitive): comment, ecnumber,
        keyword, name, pathway, term, xrefdb, xrefid, dataSource, and organism.

        The pathway field maps to all participants of pathways that contain
        the keyword(s) in any of its text fields.

        Finally, keyword is a transitive aggregate field that includes all
        searchable keywords of that element and its child elements.

        All searches can also be filtered by data source and organism.

        It is also possible to restrict the domain class using the
        'type' parameter.

        This query can be used standalone or to retrieve starting points
        for graph searches.


        :param str q: requires a keyword , name, external identifier, or a
            Lucene query string.
        :param int page: (N>=0, default is 0), search result page number.
        :param str datasource: filter by data source (use names or URIs of
            pathway data sources or of any existing Provenance object). If
            multiple data source values are specified, a union of hits from
            specified sources is returned. datasource=[reactome,pid] returns
            hits associated with Reactome or PID.
        :param str organism: The organism can be specified either by
            official name, e.g. "homo sapiens" or by NCBI taxonomy id,
            e.g. "9606". Similar to data sources, if multiple organisms
            are declared a union of all hits from specified organisms
            is returned. For example organism=[9606, 10016] returns results
            for both human and mice.
        :param str type: BioPAX class filter


        .. doctest::

            >>> from bioservices import PathwayCommons
            >>> pc2 = PathwayCommons(vverbose=False)
            >>> pc2.search("Q06609")
            >>> pc2.search("brca2", type="proteinreference",
                    organism="homo sapiens",  datasource="pid")
            >>> pc2.search("name:'col5a1'", type="proteinreference", organism=9606)
            >>> pc2.search("a*", page=3)

        """
        if self.default_extension == "xml":
            url = "search.xml?q=%s"  % q
        elif self.default_extension == "json":
            url = "search.json?q=%s"  % q

        params = {}
        if page>0:
            params['page'] = page
        else:
            self.logging.warning("page should be >=0")

        if datasource:
            params['datasource'] = datasource

        if type:
            params['type'] = type

        if organism:
            params['organism'] = organism

        res = self.http_get(url, frmt=self.default_extension,
                params=params)

        #if self.default_extension == "json":
        #    res = json.loads(res)
        if self.default_extension == "xml":
            res = self.easyXML(res)

        return res


    def get(self, uri, frmt="BIOPAX"):
        """Retrieves full pathway information for a set of elements

        elements can be for example pathway, interaction or physical
        entity given the RDF IDs. Get commands only
        retrieve the BioPAX elements that are directly mapped to the ID.
        Use the :meth:`traverse` query to traverse BioPAX graph and
        obtain child/owner elements.

        :param str uri: valid/existing BioPAX element's URI (RDF ID; for
            utility classes that were "normalized", such as entity refereneces
            and controlled vocabularies, it is usually a Identifiers.org URL.
            Multiple IDs can be provided using list
            uri=[http://identifiers.org/uniprot/Q06609,
            http://identifiers.org/uniprot/Q549Z0']
            See also about MIRIAM and Identifiers.org.
        :param str format: output format (values)

        :return: a complete BioPAX representation for the record
            pointed to by the given URI is returned. Other output
            formats are produced by converting the BioPAX record on
            demand and can be specified by the optional format
            parameter. Please be advised that with some output formats
            it might return "no result found" error if the conversion is
            not applicable for the BioPAX result. For example,
            BINARY_SIF output usually works if there are some
            interactions, complexes, or pathways in the retrieved set
            and not only physical entities.


        .. doctest::

            >>> from bioservices import PathwayCommons
            >>> pc2 = PathwayCommons(verbose=False)
            >>> res = pc2.get("col5a1")
            >>> res = pc2.get("http://identifiers.org/uniprot/Q06609")

        """
        self.devtools.check_param_in_list(frmt, self._valid_format)

        # validates the URIs
        if isinstance(uri, str):
            url = "get?uri=" +uri
        elif instance(uri, list):
            url = "get?uri=" +uri[0]
            if len(uri)>1:
                for u in uri[1:]:
                    url += "&uri=" + u

        # ?uri=http://identifiers.org/uniprot/Q06609
        # http://www.pathwaycommons.org/pc2/get?uri=COL5A1

        if frmt != "BIOPAX":
            url += "&format=%s" % frmt

        res = self.http_get(url, frmt=None)

        return res


    def top_pathways(self, datasource=None, organism=None):
        """This command returns all *top* pathways

        Pathways can be top or pathways that are neither
        'controlled' nor 'pathwayComponent' of another process.

        :param str datasource: filter by data source (same as search)
        :param str organism: organism filter

        :return: dictionary with information about top pathways. Check the
            "searchHit" key for information about "dataSource" for instance


        .. doctest::

            >>> from bioservices import PathwayCommons
            >>> pc2 = PathwayCommons(verbose=False)
            >>> res = pc2.top_pathways()



        """
        if self.default_extension == "json":
            url = "top_pathways.json"
        else:
            url = "top_pathways"

        params = {}
        if datasource:
            params['datasource'] = datasource
        if organism:
            params['organism'] = organism

        # version 1.4.3 not required anymore 
        #if len(params):
        #    url += "&" + self.urlencode(params)

        res = self.http_get(url, frmt=self.default_extension,
                params=params)

        if self.default_extension == "xml":
            res = self.easyXML(res)
        return res



    def idmapping(self, ids):
        """Identifier mapping tool

        Unambiguously maps, e.g., HGNC gene symbols, NCBI Gene, RefSeq, ENS,
        and secondary UniProt identifiers to the primary UniProt accessions,
        or - ChEBI and PubChem IDs to primary ChEBI. You can mix different
        standard ID types in one query.

        .. note:: this is a specific id-mapping (not general-purpose) for
            reference proteins and small molecules; the mapping tables
            were derived exclusively from Swiss-Prot (DR fields) and
            ChEBI data

        :param str ids: list of Identifiers or a single identifier string.
        :return: a dictionary

        .. doctest::

            >>> from bioservices import PathwayCommons
            >>> pc2 = PathwayCommons(verbose=False)
            >>> pc2.idmapping("BRCA2")
            {u'BRCA2': u'P51587'}
            >>> pc2.idmapping(["TP53", "BRCA2"])
            {"BRCA2":"P51587","TP53":"P04637"}


        """
        url = "idmapping?id="
        if isinstance(ids, str):
            url += ids
        elif isinstance(ids, list):
            url += ids[0]
            if len(ids) > 1:
                for id_ in ids[1:]:
                    url += "&id=" + id_
        res = self.http_get(url, frmt="json")
        #res = json.loads(res)
        return res

    def graph(self, kind, source, target=None, direction=None, limit=1,
            frmt=None, datasource=None, organism=None):
        """Finds connections and neighborhoods of elements

        Connections can be for example the shortest path between two proteins
        or the neighborhood for a particular protein state or all states.

        Graph searches take detailed BioPAX semantics such as generics or
        nested complexes into account and traverse the graph accordingly.
        The starting points can be either physical entites or entity references.

        In the case of the latter the graph search starts from ALL
        the physical entities that belong to that particular entity references,
        i.e.  all of its states. Note that we integrate BioPAX data from
        multiple databases  based on our proteins and small molecules data
        warehouse and consistently normalize UnificationXref, EntityReference,
        Provenance, BioSource, and ControlledVocabulary objects when we are
        absolutely sure that two objects of the same type are equivalent. We,
        however, do not merge physical entities and reactions from different
        sources as matching and aligning pathways at that level is still an
        open research problem. As a result, graph searches can return
        several similar but disconnected sub-networks that correspond to
        the pathway data from different providers (though some physical
        entities often refer to the same small molecule or protein reference
        or controlled vocabulary).


        :param str kind: graph query
        :param str source:  source object's URI/ID. Multiple source URIs/IDs
            must be encoded as list of valid URI
            **source=['http://identifiers.org/uniprot/Q06609',
            'http://identifiers.org/uniprot/Q549Z0']**.
        :param str target: required for PATHSFROMTO graph query.  target
            URI/ID. Multiple target URIs must be encoded as list (see source
            parameter).
        :param str direction: graph search  direction in [BOTHSTREAM,
            DOWNSTREAM, UPSTREAM] see :attr:`_valid_direction` attribute.
        :param int limit: graph query search distance limit (default = 1).
        :param str format: output format. see :attr:`_valid-format`
        :param str datasource: datasource filter (same as for 'search').
        :param str organism: organism filter (same as for 'search').


        :return:  By default, graph queries return a complete BioPAX
            representation of the subnetwork matched by the algorithm.
            Other output formats are available as specified by the optional
            format parameter. Please be advised that some output format
            choices might cause "no result found" error if the conversion
            is not applicable for the BioPAX result (e.g., BINARY_SIF output
            fails if there are no interactions, complexes, nor pathways
            in the retrieved set).

        .. doctest::

            >>> from bioservices import PathwayCommons
            >>> pc2 = PathwayCommons(verbose=False)
            >>> res = pc2.graph(source="http://identifiers.org/uniprot/P20908",
                    kind="neighborhood", format="EXTENDED_BINARY_SIF")



        """
        url = "graph"
        params = {}
        params['source'] = source
        params['kind'] = kind
        params['limit'] = limit

        params = {}
        if target:
            params['target'] = target
        if frmt:
            params['format'] = frmt
        if datasource:
            params['datasource'] = datasource
        if organism:
            params['organism'] = organism

        res = self.http_get(url, frmt=None, params=params)
        return res

    def traverse(self, uri, path):
        """Provides XPath-like access to the PC.


        The format of the path query is in the form::

            [InitialClass]/[property1]:[classRestriction(optional)]/[property2]... A "*"

        sign after the property instructs path accessor to transitively traverse
        that property. For example, the following path accessor will traverse
        through all physical entity components within a complex::

            "Complex/component*/entityReference/xref:UnificationXref"

        The following will list display names of all participants of
        interactions, which are components (pathwayComponent) of a pathway
        (note: pathwayOrder property, where same or other interactions can be
        reached, is not considered here)::

            "Pathway/pathwayComponent:Interaction/participant*/displayName"

        The optional parameter classRestriction allows to restrict/filter the
        returned property values to a certain subclass of the range of that
        property. In the first example above, this is used to get only the
        Unification Xrefs. Path accessors can use all the official BioPAX
        properties as well as additional derived classes and parameters in
        paxtools such as inverse parameters and interfaces that represent
        anonymous union classes in OWL. (See Paxtools documentation for more
        details).

        :param str uri: a biopax element URI - specified similar to the 'GET'
            command. multiple IDs are allowed as a list of strings.
        :param str path: a BioPAX propery path in the form of
                property1[:type1]/property2[:type2]; see above, inverse
                properties, Paxtools,
                org.biopax.paxtools.controller.PathAccessor.

        .. seealso:: `properties
            <http://www.pathwaycommons.org/pc2/#biopax_properties>`_

        :return:  XML result that follows the Search Response XML Schema
            (TraverseResponse type; pagination is disabled: returns all values at
            once)

        ::


            from bioservices import PathwayCommons
            pc2 = PathwayCommons(verbose=False)
            res = pc2.traverse(uri=['http://identifiers.org/uniprot/P38398','http://identifiers.org/uniprot/Q06609'], path="ProteinReference/organism")
            res = pc2.traverse(uri="http://identifiers.org/uniprot/Q06609",
                path="ProteinReference/entityReferenceOf:Protein/name")
            res = pc2.traverse("http://identifiers.org/uniprot/P38398",
                path="ProteinReference/entityReferenceOf:Protein")
            res = pc2.traverse(uri=["http://identifiers.org/uniprot/P38398",
                "http://identifiers.org/taxonomy/9606"], path="Named/name")


        """
        url =  "traverse?"

        if isinstance(uri, str):
            url += "?uri=" + uri
        elif isinstance(uri, list):
            url += "?uri=" + uri[0]
            for u in uri[1:]:
                url += "&uri=" + u

        url += "&path=" + path

        res = self.http_get(url, frmt=None)
        return res





# -*- coding: utf-8 -*-
"""
Created on Fri Aug  8 15:31:34 2014

@author: cokelaer
"""
import os
from easydev import DynamicConfigParser, underline
import copy
import shutil

import appdirs

__all__ = ["defaultParams", "BioServicesConfig"]


#TODO Move some contents to easydev.config_tools

# first item if the value
# second item if a type or TUPLE of types possible
# third item is documentation
defaultParams = {
    'user.email': ["unknown", (str), "email addresss that may be used in some utilities (e.g. EUtils)"],
    'general.timeout': [30, (int,float), ""],
    'general.max_retries': [3, int, ''],
    'general.async_concurrent': [50, int, ''],
    'general.async_threshold': [10, int, 'when to switch to asynchronous requests'],
    'cache.tag_suffix': ["_bioservices_database",str, 'suffix to append for cache databases'],
    'cache.on': [False, bool, 'CACHING on/off'],
    'cache.fast': [True, bool, "FAST_SAVE option"],
    'chemspider.token': [None, (str, type(None)), 'token see http://www.chemspider.com'],
}


class ConfigReadOnly(object):
    """A generic Config file handler

    Uses appdirs from ypi to handle the XDG protocol

    Read the configuration in the XDG directory. If not found, the
    config and cache directories are created. Then, reads the configuration
    file. If not found, nothing happens. A dictionary should be provided
    to initialise the default parameters. This dictionary is updated
    wit the content of the user config file if any. Reset the parameters
    to the default values is possible at any time. Re-read the user
    config file is possible at any time (meth:`read_`)

    """
    def __init__(self, name=None, default_params={}):
        """name is going to be the generic name of the config folder

        e.g., /home/user/.config/<name>/<name>.cfg

        """
        if name is None:
            raise Exception("Name parameter must be provided")
        else:
            # use input parameters
            self.name = name
            self._default_params = copy.deepcopy(default_params)
            self.params = copy.deepcopy(default_params)

            # useful tool to handle XDG config file, path and parameters
            self.appdirs = appdirs.AppDirs(self.name)

            # useful tool to handle the config ini file
            self.config_parser = DynamicConfigParser()

            # Now, create the missing directories if needed
            self.init() # and read the user config file updating params if needed

    def read_user_config_file_and_update_params(self):
        """Read the configuration file and update parameters

        Read the configuration file (file with extension cfg and name of your
        app). Section and option found will be used to update the :attr:`params`.

        If a set of section/option is not correct (not in the :attr:`params`) then
        it is ignored.

        The :attr:`params` is a dictionary with keys being labelled as <section>.<option>
        For instance, the key "cache.on" should be written in the configuration file as::

            [cache]
            on = True

        where True is the expected value.


        """
        try:
            self.config_parser.read(self.user_config_file_path)
        except IOError:

            msg = "Welcome to %s" % self.name.capitalize()
            print(underline(msg))
            print("It looks like you do not have a configuration file.")
            print("We are creating one with default values in %s ." % self.user_config_file_path)
            print("Done")
            self.create_default_config_file()

        # now, update the params attribute if needed
        for section in self.config_parser.sections():
            dic = self.config_parser.section2dict(section)
            for key in dic.keys():
                value = dic[key]
                newkey = section + '.' + key
                if newkey in self.params.keys():
                    # the type should be self.params[newkey][1]
                    cast = self.params[newkey][1]
                    # somehow
                    if isinstance(value, cast) is True:
                        self.params[newkey][0] = value
                    else:
                        print("Warning:: found an incorrect type while parsing {} file. In section '{}', the option '{}' should be a {}. Found value {}. Trying a cast...".format(self.user_config_file_path, section, key, cast, value))
                        self.params[newkey][0] = cast(value)
                else:
                    print("Warning:: found invalid option or section in %s (ignored):" % self.user_config_file_path)
                    print("   %s %s" % (section, option))

    def _get_home(self):
        # This function should be robust
        # First, let us try with expanduser
        try:
            homedir = os.path.expanduser("~")
        except ImportError:
            # This may happen.
            pass
        else:
            if os.path.isdir(homedir):
                return homedir
        # Then, with getenv
        for this in ('HOME', 'USERPROFILE', 'TMP'):
            # getenv is same as os.environ.get
            homedir = os.environ.get(this)
            if homedir is not None and os.path.isdir(homedir):
                return homedir
        return None
    home = property(_get_home)

    def _mkdirs(self, newdir, mode=0o777):
        """from matplotlib mkdirs

        make directory *newdir* recursively, and set *mode*.  Equivalent to ::

        > mkdir -p NEWDIR
        > chmod MODE NEWDIR
        """
        try:
            if not os.path.exists(newdir):
                parts = os.path.split(newdir)
                for i in range(1, len(parts) + 1):
                    thispart = os.path.join(*parts[:i])
                    if not os.path.exists(thispart):
                        os.makedirs(thispart, mode)

        except OSError as err:
            # Reraise the error unless it's about an already existing directory
            if err.errno != errno.EEXIST or not os.path.isdir(newdir):
                raise

    def _get_and_create(self, sdir):
        if not os.path.exists(sdir):
            print("Creating directory %s " % sdir)
            try:
                self._mkdirs(sdir)
            except Exception:
                print("Could not create the path %s " % sdir)
                return None
        return sdir

    def _get_config_dir(self):
        sdir = self.appdirs.user_config_dir
        return self._get_and_create(sdir)
    user_config_dir = property(_get_config_dir,
            doc="return directory of this configuration file")

    def _get_cache_dir(self):
        sdir = self.appdirs.user_cache_dir
        return self._get_and_create(sdir)
    user_cache_dir = property(_get_cache_dir,
            doc="return directory of the cache")

    def _get_config_file_path(self):
        return self.user_config_dir + os.sep +self.config_file
    user_config_file_path = property(_get_config_file_path,
            doc="return configuration filename (with fullpath)")

    def _get_config_file(self):
        return self.name + ".cfg"
    config_file = property(_get_config_file,
            doc="config filename (without path)")

    def init(self):
        """Reads the user_config_file and update params.
        Creates the directories for config and cache if they do not exsits

        """
        # Let us create the directories by simply getting these 2 attributes:
        try:
            _ = self.user_config_dir
        except:
            print("Could not retrieve or create the config file and/or directory in %s" % self.name)
        try:
            _ = self.user_cache_dir
        except:
            print("Could not retrieve or create the cache file and/or directory in %s" % self.name)
        self.read_user_config_file_and_update_params()

    def create_default_config_file(self, force=False):

        # if the file already exists, we should save the file into
        # a backup file
        if os.path.exists(self.user_config_file_path):
            # we need to copy the file into a backup file
            filename = self.user_config_file_path + '.bk'
            if os.path.exists(filename) and force is False:
                print("""Trying to save the current config file {} into a backup file {}\n but it exists already. Please remove the backup file first or set the 'force' parameter to True""".format(self.user_config_file_path, filename))
                return
            else:
                shutil.copy(self.user_config_file_path, filename)

        # Now, we can rewrite the configuration file
        sections = sorted(set([x.split(".")[0] for x in self._default_params.keys()]))
        if 'general' in sections:
            sections = ["general"] + [x for x in sections if x!="general"]

        fh = open(self.user_config_file_path, "w") # open and delete content
        for section in sections:
            fh.write("[" + section +"]\n")
            options = [x.split(".")[1] for x in self._default_params.keys() if x.startswith(section+".")]
            for option in options:
                key = section + '.' + option
                value = self._default_params[key]
                try:
                    fh.write("# {}\n{} = {}\n".format(value[2], 
                        option, value[0]))
                except:
                    print('Could not write this value/option. skipped')
                    print(value, option)

            fh.write("\n")
        fh.close()

    def reload_default_params(self):
        self.params = copy.deepcopy(self._default_params)



class BioServicesConfig(ConfigReadOnly):
    def __init__(self):
        super(BioServicesConfig, self).__init__(name="bioservices",
                default_params=defaultParams)

    # some aliases
    def _get_caching(self):
        return self.params['cache.on'][0]
    def _set_caching(self, value):
        self.params['cache.on'][0] = value
    CACHING = property(_get_caching)

    def _get_fast_save(self):
        return self.params['cache.fast'][0]
    FAST_SAVE = property(_get_fast_save)

    def _get_async_concurrent(self):
        return self.params['general.async_concurrent'][0]
    CONCURRENT = property(_get_async_concurrent)

    def _get_async_threshold(self):
        return self.params['general.async_threshold'][0]
    ASYNC_THRESHOLD = property(_get_async_threshold)

    def _get_timeout(self):
        return self.params['general.timeout'][0]
    def _set_timeout(self, timeout):
        self.params['general.timeout'][0] = timeout
    TIMEOUT = property(_get_timeout, _set_timeout)

    def _get_max_retries(self):
        return self.params['general.max_retries'][0]
    def _set_max_retries(self, max_retries):
        self.params['general.max_retries'][0] = max_retries
    MAX_RETRIES = property(_get_max_retries, _set_max_retries)
